@@ -21,43 +21,50 @@ import json
 import logging
 import re
 import sys
-import traceback
 import uuid
+import traceback
 from collections import namedtuple
 from concurrent.futures import Future
-from functools import partial
+from functools import lru_cache, partial
 from itertools import zip_longest
-from typing import Callable, List, Optional
+from typing import Any, Callable, List, Optional, Type, Union
+
+import attrs
+from cattrs.errors import ClassValidationError
+from lsprotocol import converters
 
 from pygls.capabilities import ServerCapabilitiesBuilder
 from pygls.constants import ATTR_FEATURE_TYPE
-from pygls.exceptions import (JsonRpcException, JsonRpcInternalError, JsonRpcInvalidParams,
-                              JsonRpcMethodNotFound, JsonRpcRequestCancelled,
-                              MethodTypeNotRegisteredError)
-from pygls.feature_manager import (FeatureManager, assign_help_attrs, get_help_attrs,
-                                   is_thread_function)
-from pygls.lsp import (JsonRPCNotification, JsonRPCRequestMessage, JsonRPCResponseMessage,
-                       get_method_params_type, get_method_return_type, is_instance)
-from pygls.lsp.methods import (CANCEL_REQUEST, CLIENT_REGISTER_CAPABILITY,
-                               CLIENT_UNREGISTER_CAPABILITY, EXIT, INITIALIZE, INITIALIZED,
-                               LOG_TRACE_NOTIFICATION, SET_TRACE_NOTIFICATION, SHUTDOWN,
-                               TEXT_DOCUMENT_DID_CHANGE, TEXT_DOCUMENT_DID_CLOSE,
-                               TEXT_DOCUMENT_DID_OPEN, TEXT_DOCUMENT_PUBLISH_DIAGNOSTICS,
-                               WINDOW_LOG_MESSAGE, WINDOW_SHOW_DOCUMENT, WINDOW_SHOW_MESSAGE,
-                               WORKSPACE_APPLY_EDIT, WORKSPACE_CONFIGURATION,
-                               WORKSPACE_DID_CHANGE_WORKSPACE_FOLDERS, WORKSPACE_EXECUTE_COMMAND,
-                               WORKSPACE_SEMANTIC_TOKENS_REFRESH)
-from pygls.lsp.types import (ApplyWorkspaceEditParams, ApplyWorkspaceEditResponse, Diagnostic,
-                             DidChangeTextDocumentParams, DidChangeWorkspaceFoldersParams,
-                             DidCloseTextDocumentParams, DidOpenTextDocumentParams,
-                             ExecuteCommandParams, InitializeParams, InitializeResult,
-                             LogMessageParams, MessageType, PublishDiagnosticsParams,
-                             RegistrationParams, ShowMessageParams, UnregistrationParams,
-                             WorkspaceEdit)
-from pygls.lsp.types.basic_structures import (ConfigCallbackType, LogTraceParams, SetTraceParams,
-                                              Trace)
-from pygls.lsp.types.window import ShowDocumentCallbackType, ShowDocumentParams
-from pygls.lsp.types.workspace import ConfigurationParams
+from pygls.exceptions import (
+    JsonRpcException, JsonRpcInternalError, JsonRpcInvalidParams,
+    JsonRpcMethodNotFound, JsonRpcRequestCancelled,
+    FeatureNotificationError, FeatureRequestError
+)
+from pygls.feature_manager import FeatureManager, assign_help_attrs, is_thread_function
+from pygls.lsp import ConfigCallbackType, ShowDocumentCallbackType
+from lsprotocol.types import (
+    CANCEL_REQUEST, CLIENT_REGISTER_CAPABILITY,
+    CLIENT_UNREGISTER_CAPABILITY, EXIT, INITIALIZE, INITIALIZED,
+    METHOD_TO_TYPES, LOG_TRACE, SET_TRACE, SHUTDOWN,
+    TEXT_DOCUMENT_DID_CHANGE, TEXT_DOCUMENT_DID_CLOSE,
+    TEXT_DOCUMENT_DID_OPEN, TEXT_DOCUMENT_PUBLISH_DIAGNOSTICS,
+    WINDOW_LOG_MESSAGE, WINDOW_SHOW_DOCUMENT, WINDOW_SHOW_MESSAGE,
+    WORKSPACE_APPLY_EDIT, WORKSPACE_CONFIGURATION,
+    WORKSPACE_DID_CHANGE_WORKSPACE_FOLDERS, WORKSPACE_EXECUTE_COMMAND,
+    WORKSPACE_SEMANTIC_TOKENS_REFRESH
+)
+from lsprotocol.types import (
+    ApplyWorkspaceEditParams,
+    ConfigurationParams, Diagnostic,
+    DidChangeTextDocumentParams, DidChangeWorkspaceFoldersParams,
+    DidCloseTextDocumentParams, DidOpenTextDocumentParams,
+    ExecuteCommandParams, InitializeParams, InitializeResult,
+    LogMessageParams, LogTraceParams, MessageType, PublishDiagnosticsParams,
+    RegistrationParams, ResponseErrorMessage, SetTraceParams,
+    ShowDocumentParams, ShowMessageParams,
+    TraceValues, UnregistrationParams, WorkspaceApplyEditResponse,
+    WorkspaceEdit, InitializeResultServerInfoType
+)
 from pygls.uris import from_fs_path
 from pygls.workspace import Workspace
 
@@ -86,8 +93,49 @@ def call_user_feature(base_func, method_name):
     return decorator
 
 
-def dict_to_object(**d):
+@attrs.define
+class JsonRPCNotification:
+    """A class that represents a generic json rpc notification message.
+    Used as a fallback for unknown types.
+    """
+
+    method: str
+    jsonrpc: str
+    params: Any
+
+
+@attrs.define
+class JsonRPCRequestMessage:
+    """A class that represents a generic json rpc request message.
+    Used as a fallback for unknown types.
+    """
+
+    id: Union[int, str]
+    method: str
+    jsonrpc: str
+    params: Any
+
+
+@attrs.define
+class JsonRPCResponseMessage:
+    """A class that represents a generic json rpc response message.
+    Used as a fallback for unknown types.
+    """
+
+    id: Union[int, str]
+    jsonrpc: str
+    result: Any
+
+
+def _dict_to_object(d: Any):
     """Create nested objects (namedtuple) from dict."""
+
+    if d is None:
+        return None
+
+    if not isinstance(d, dict):
+        return d
+
     type_name = d.pop('type_name', 'Object')
     return json.loads(
         json.dumps(d),
@@ -95,73 +143,37 @@ def dict_to_object(**d):
     )
 
 
-def default_serializer(o):
-    """JSON serializer for complex objects that do not extend pydantic BaseModel class."""
-    if isinstance(o, enum.Enum):
-        return o.value
-    return o.__dict__
+def _params_field_structure_hook(obj, cls):
+    if 'params' in obj:
+        obj['params'] = _dict_to_object(obj['params'])
+
+    return cls(**obj)
 
 
-def deserialize_command(params):
-    """Function used to deserialize command arguments to a specific class
-    or a namedtuple."""
-    # TODO: Register/Look up custom command arguments' types
-    # Currently command parameters are type of 'any', but eventually we would
-    # want to register an argument type of our custom command and to
-    # deserialize it properly.
-    temp_obj = dict_to_object(**params, type_name='CommandParams')
+def _result_field_structure_hook(obj, cls):
+    if 'result' in obj:
+        obj['result'] = _dict_to_object(obj['result'])
 
-    params['arguments'] = getattr(temp_obj, 'arguments', None)
-    return params
+    return cls(**obj)
 
 
-def deserialize_params(data, get_params_type):
-    """Function used to deserialize params to a specific class."""
-    try:
-        method = data['method']
-        params = data['params']
+def default_converter():
+    """Default converter factory function."""
 
-        if not isinstance(params, dict):
-            return data
+    converter = converters.get_converter()
+    converter.register_structure_hook(
+        JsonRPCRequestMessage, _params_field_structure_hook
+    )
 
-        try:
-            params_type = get_params_type(method)
-            if params_type is None:
-                params_type = dict_to_object
-            elif params_type.__name__ == ExecuteCommandParams.__name__:
-                params = deserialize_command(params)
+    converter.register_structure_hook(
+        JsonRPCResponseMessage, _result_field_structure_hook
+    )
 
-        except MethodTypeNotRegisteredError:
-            params_type = dict_to_object
+    converter.register_structure_hook(
+        JsonRPCNotification, _params_field_structure_hook
+    )
 
-        try:
-            data['params'] = params_type(**params)
-        except TypeError:
-            raise ValueError(
-                f'Could not instantiate "{params_type.__name__}" from params: {params}')
-    except KeyError:
-        pass
-
-    return data
-
-
-def deserialize_message(data, get_params_type=get_method_params_type):
-    """Function used to deserialize data received from client."""
-    if 'jsonrpc' in data:
-        try:
-            deserialize_params(data, get_params_type)
-        except ValueError:
-            raise JsonRpcInvalidParams()
-
-        if 'id' in data:
-            if 'method' in data:
-                return JsonRPCRequestMessage(**data)
-            else:
-                return JsonRPCResponseMessage(**data)
-        else:
-            return JsonRPCNotification(**data)
-
-    return data
+    return converter
 
 
 class JsonRPCProtocol(asyncio.Protocol):
@@ -185,12 +197,15 @@ class JsonRPCProtocol(asyncio.Protocol):
 
     VERSION = '2.0'
 
-    def __init__(self, server):
+    def __init__(self, server, converter):
         self._server = server
+        self._converter = converter
+
         self._shutdown = False
 
-        self._client_request_futures = {}
-        self._server_request_futures = {}
+        # Book keeping for in-flight requests
+        self._request_futures = {}
+        self._result_types = {}
 
         self.fm = FeatureManager(server)
         self.transport = None
@@ -201,16 +216,6 @@ class JsonRPCProtocol(asyncio.Protocol):
     def __call__(self):
         return self
 
-    def _check_ret_type_and_send_response(self, method_name, method_type, msg_id, result):
-        """Check if registered feature returns appropriate result type."""
-        if method_type == ATTR_FEATURE_TYPE:
-            return_type = get_method_return_type(method_name)
-            if not is_instance(result, return_type):
-                error = JsonRpcInternalError().to_dict()
-                self._send_response(msg_id, error=error)
-
-        self._send_response(msg_id, result=result)
-
     def _execute_notification(self, handler, *params):
         """Executes notification message handler."""
         if asyncio.iscoroutinefunction(handler):
@@ -238,39 +243,34 @@ class JsonRPCProtocol(asyncio.Protocol):
 
     def _execute_request(self, msg_id, handler, params):
         """Executes request message handler."""
-        method_name, method_type = get_help_attrs(handler)
 
         if asyncio.iscoroutinefunction(handler):
             future = asyncio.ensure_future(handler(params))
-            self._client_request_futures[msg_id] = future
-            future.add_done_callback(partial(self._execute_request_callback,
-                                             method_name, method_type, msg_id))
+            self._request_futures[msg_id] = future
+            future.add_done_callback(partial(self._execute_request_callback, msg_id))
         else:
             # Can't be canceled
             if is_thread_function(handler):
                 self._server.thread_pool.apply_async(
                     handler, (params, ),
                     callback=partial(
-                        self._check_ret_type_and_send_response,
-                        method_name, method_type, msg_id,
+                        self._send_response, msg_id,
                     ),
                     error_callback=partial(self._execute_request_err_callback, msg_id))
             else:
-                self._check_ret_type_and_send_response(
-                    method_name, method_type, msg_id, handler(params))
+                self._send_response(msg_id, handler(params))
 
-    def _execute_request_callback(self, method_name, method_type, msg_id, future):
+    def _execute_request_callback(self, msg_id, future):
         """Success callback used for coroutine request message."""
         try:
             if not future.cancelled():
-                self._check_ret_type_and_send_response(
-                    method_name, method_type, msg_id, result=future.result())
+                self._send_response(msg_id, result=future.result())
             else:
                 self._send_response(
                     msg_id,
                     error=JsonRpcRequestCancelled(f'Request with id "{msg_id}" is canceled')
                 )
-            self._client_request_futures.pop(msg_id, None)
+            self._request_futures.pop(msg_id, None)
         except Exception:
             error = JsonRpcInternalError.of(sys.exc_info()).to_dict()
             logger.exception('Exception occurred for message "%s": %s', msg_id, error)
@@ -295,7 +295,7 @@ class JsonRPCProtocol(asyncio.Protocol):
 
     def _handle_cancel_notification(self, msg_id):
         """Handles a cancel notification from the client."""
-        future = self._client_request_futures.pop(msg_id, None)
+        future = self._request_futures.pop(msg_id, None)
 
         if not future:
             logger.warning('Cancel notification for unknown message id "%s"', msg_id)
@@ -316,8 +316,14 @@ class JsonRPCProtocol(asyncio.Protocol):
             self._execute_notification(handler, params)
         except (KeyError, JsonRpcMethodNotFound):
             logger.warning('Ignoring notification for unknown method "%s"', method_name)
-        except Exception:
-            logger.exception('Failed to handle notification "%s": %s', method_name, params)
+        except Exception as error:
+            logger.exception(
+                'Failed to handle notification "%s": %s',
+                method_name,
+                params,
+                exc_info=True
+            )
+            self._server._report_server_error(error, FeatureNotificationError)
 
     def _handle_request(self, msg_id, method_name, params):
         """Handles a request from the client."""
@@ -330,17 +336,31 @@ class JsonRPCProtocol(asyncio.Protocol):
             else:
                 self._execute_request(msg_id, handler, params)
 
-        except JsonRpcException as e:
-            logger.exception('Failed to handle request %s %s %s', msg_id, method_name, params)
-            self._send_response(msg_id, None, e.to_dict())
-        except Exception:
-            logger.exception('Failed to handle request %s %s %s', msg_id, method_name, params)
+        except JsonRpcException as error:
+            logger.exception(
+                'Failed to handle request %s %s %s',
+                msg_id,
+                method_name,
+                params,
+                exc_info=True
+            )
+            self._send_response(msg_id, None, error.to_dict())
+            self._server._report_server_error(error, FeatureRequestError)
+        except Exception as error:
+            logger.exception(
+                'Failed to handle request %s %s %s',
+                msg_id,
+                method_name,
+                params,
+                exc_info=True
+            )
             err = JsonRpcInternalError.of(sys.exc_info()).to_dict()
             self._send_response(msg_id, None, err)
+            self._server._report_server_error(error, FeatureRequestError)
 
     def _handle_response(self, msg_id, result=None, error=None):
         """Handles a response from the client."""
-        future = self._server_request_futures.pop(msg_id, None)
+        future = self._request_futures.pop(msg_id, None)
 
         if not future:
             logger.warning('Received response to unknown message id "%s"', msg_id)
@@ -348,13 +368,59 @@ class JsonRPCProtocol(asyncio.Protocol):
 
         if error is not None:
             logger.debug('Received error response to message "%s": %s', msg_id, error)
-            future.set_exception(JsonRpcException.from_dict(error))
+            future.set_exception(JsonRpcException.from_error(error))
         else:
             logger.debug('Received result for message "%s": %s', msg_id, result)
             future.set_result(result)
 
+    def _serialize_message(self, data):
+        """Function used to serialize data sent to the client."""
+
+        if hasattr(data, '__attrs_attrs__'):
+            return self._converter.unstructure(data)
+
+        if isinstance(data, enum.Enum):
+            return data.value
+
+        return data.__dict__
+
+    def _deserialize_message(self, data):
+        """Function used to deserialize data recevied from the client."""
+
+        if 'jsonrpc' not in data:
+            return data
+
+        try:
+            if 'id' in data:
+                if 'error' in data:
+                    return self._converter.structure(data, ResponseErrorMessage)
+                elif 'method' in data:
+                    request_type = (
+                        self.get_message_type(data['method']) or JsonRPCRequestMessage
+                    )
+                    return self._converter.structure(data, request_type)
+                else:
+                    response_type = (
+                        self._result_types.pop(data['id']) or JsonRPCResponseMessage
+                    )
+                    return self._converter.structure(data, response_type)
+
+            else:
+                method = data.get('method', '')
+                notification_type = self.get_message_type(method) or JsonRPCNotification
+                return self._converter.structure(data, notification_type)
+
+        except ClassValidationError as exc:
+            logger.error("Unable to deserialize message\n%s", traceback.format_exc())
+            raise JsonRpcInvalidParams() from exc
+
+        except Exception as exc:
+            logger.error("Unable to deserialize message\n%s", traceback.format_exc())
+            raise JsonRpcInternalError() from exc
+
     def _procedure_handler(self, message):
         """Delegates message to handlers depending on message type."""
+
         if message.jsonrpc != JsonRPCProtocol.VERSION:
             logger.warning('Unknown message "%s"', message)
             return
@@ -363,23 +429,32 @@ class JsonRPCProtocol(asyncio.Protocol):
             logger.warning('Server shutting down. No more requests!')
             return
 
-        if isinstance(message, JsonRPCNotification):
-            logger.debug('Notification message received.')
-            self._handle_notification(message.method, message.params)
-        elif isinstance(message, JsonRPCResponseMessage):
-            logger.debug('Response message received.')
-            self._handle_response(message.id, message.result, message.error)
-        elif isinstance(message, JsonRPCRequestMessage):
-            logger.debug('Request message received.')
-            self._handle_request(message.id, message.method, message.params)
+        if hasattr(message, 'method'):
+            if hasattr(message, 'id'):
+                logger.debug('Request message received.')
+                self._handle_request(message.id, message.method, message.params)
+            else:
+                logger.debug('Notification message received.')
+                self._handle_notification(message.method, message.params)
+        else:
+            if hasattr(message, 'error'):
+                logger.debug('Error message received.')
+                self._handle_response(message.id, None, message.error)
+            else:
+                logger.debug('Response message received.')
+                self._handle_response(message.id, message.result)
 
     def _send_data(self, data):
         """Sends data to the client."""
         if not data:
             return
 
+        if self.transport is None:
+            logger.error("Unable to send data, no available transport!")
+            return
+
         try:
-            body = data.json(by_alias=True, exclude_unset=True, encoder=default_serializer)
+            body = json.dumps(data, default=self._serialize_message)
             logger.info('Sending data: %s', body)
 
             body = body.encode(self.CHARSET)
@@ -392,8 +467,9 @@ class JsonRPCProtocol(asyncio.Protocol):
                 self.transport.write(header + body)
             else:
                 self.transport.write(body.decode('utf-8'))
-        except Exception:
-            logger.error(traceback.format_exc())
+        except Exception as error:
+            logger.exception("Error sending data", exc_info=True)
+            self._server._report_server_error(error, JsonRpcInternalError)
 
     def _send_response(self, msg_id, result=None, error=None):
         """Sends a JSON RPC response to the client.
@@ -403,15 +479,15 @@ class JsonRPCProtocol(asyncio.Protocol):
             result(any): Result returned by handler
             error(any): Error returned by handler
         """
-        response = JsonRPCResponseMessage(id=msg_id,
-                                          jsonrpc=JsonRPCProtocol.VERSION,
-                                          result=result,
-                                          error=error)
 
-        if error is None:
-            del response.error
+        if error is not None:
+            response = ResponseErrorMessage(id=msg_id, error=error)
+
         else:
-            del response.result
+            response_type = self._result_types.pop(msg_id, JsonRPCResponseMessage)
+            response = response_type(
+                id=msg_id, result=result, jsonrpc=JsonRPCProtocol.VERSION
+            )
 
         self._send_data(response)
 
@@ -427,6 +503,13 @@ class JsonRPCProtocol(asyncio.Protocol):
         self.transport = transport
 
     def data_received(self, data: bytes):
+        try:
+            self._data_received(data)
+        except Exception as error:
+            logger.exception("Error receiving data", exc_info=True)
+            self._server._report_server_error(error, JsonRpcInternalError)
+
+    def _data_received(self, data: bytes):
         """Method from base class, called when server receives the data"""
         logger.debug('Received %r', data)
 
@@ -454,21 +537,31 @@ class JsonRPCProtocol(asyncio.Protocol):
             # Parse the body
             self._procedure_handler(
                 json.loads(body.decode(self.CHARSET),
-                           object_hook=deserialize_message))
+                           object_hook=self._deserialize_message))
+
+    def get_message_type(self, method: str) -> Optional[Type]:
+        """Return the type definition of the message associated with the given method."""
+        return None
+
+    def get_result_type(self, method: str) -> Optional[Type]:
+        """Return the type definition of the result associated with the given method."""
+        return None
 
     def notify(self, method: str, params=None):
         """Sends a JSON RPC notification to the client."""
-        logger.debug('Sending notification: "%s" %s', method, params)
 
-        request = JsonRPCNotification(
-            jsonrpc=JsonRPCProtocol.VERSION,
+        logger.debug("Sending notification: '%s' %s", method, params)
+
+        notification_type = self.get_message_type(method) or JsonRPCNotification
+        notification = notification_type(
             method=method,
-            params=params
+            params=params,
+            jsonrpc=JsonRPCProtocol.VERSION
         )
 
-        self._send_data(request)
+        self._send_data(notification)
 
-    def send_request(self, method, params=None, callback=None):
+    def send_request(self, method, params=None, callback=None, msg_id=None):
         """Sends a JSON RPC request to the client.
 
         Args:
@@ -478,14 +571,18 @@ class JsonRPCProtocol(asyncio.Protocol):
         Returns:
             Future that will be resolved once a response has been received
         """
-        msg_id = str(uuid.uuid4())
+
+        if msg_id is None:
+            msg_id = str(uuid.uuid4())
+
+        request_type = self.get_message_type(method) or JsonRPCRequestMessage
         logger.debug('Sending request with id "%s": %s %s', msg_id, method, params)
 
-        request = JsonRPCRequestMessage(
+        request = request_type(
             id=msg_id,
-            jsonrpc=JsonRPCProtocol.VERSION,
             method=method,
-            params=params
+            params=params,
+            jsonrpc=JsonRPCProtocol.VERSION,
         )
 
         future = Future()
@@ -497,7 +594,9 @@ class JsonRPCProtocol(asyncio.Protocol):
                 callback(result)
             future.add_done_callback(wrapper)
 
-        self._server_request_futures[msg_id] = future
+        self._request_futures[msg_id] = future
+        self._result_types[msg_id] = self.get_result_type(method)
+
         self._send_data(request)
 
         return future
@@ -555,8 +654,8 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
         workspace(Workspace): In memory workspace
     """
 
-    def __init__(self, server):
-        super().__init__(server)
+    def __init__(self, server, converter):
+        super().__init__(server, converter)
 
         self.workspace = None
         self.trace = None
@@ -564,6 +663,11 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
         from pygls.progress import Progress
         self.progress = Progress(self)
 
+        self.server_info = InitializeResultServerInfoType(
+            name=server.name,
+            version=server.version,
+        )
+
         self._register_builtin_features()
 
     def _register_builtin_features(self):
@@ -573,7 +677,16 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
             if callable(attr) and hasattr(attr, 'method_name'):
                 self.fm.add_builtin_feature(attr.method_name, attr)
 
-    def apply_edit(self, edit: WorkspaceEdit, label: str = None) -> ApplyWorkspaceEditResponse:
+    @lru_cache()
+    def get_message_type(self, method: str) -> Optional[Type]:
+        """Return LSP type definitions, as provided by `lsprotocol`"""
+        return METHOD_TO_TYPES.get(method, (None,))[0]
+
+    @lru_cache()
+    def get_result_type(self, method: str) -> Optional[Type]:
+        return METHOD_TO_TYPES.get(method, (None, None))[1]
+
+    def apply_edit(self, edit: WorkspaceEdit, label: str = None) -> WorkspaceApplyEditResponse:
         """Sends apply edit request to the client."""
         return self.send_request(WORKSPACE_APPLY_EDIT,
                                  ApplyWorkspaceEditParams(edit=edit, label=label))
@@ -581,7 +694,9 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
     @lsp_method(EXIT)
     def lsp_exit(self, *args) -> None:
         """Stops the server process."""
-        self.transport.close()
+        if self.transport is not None:
+            self.transport.close()
+
         sys.exit(0 if self._shutdown else 1)
 
     @lsp_method(INITIALIZE)
@@ -603,7 +718,10 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
             list(self.fm.commands.keys()),
             self._server.sync_kind,
         ).build()
-        logger.debug('Server capabilities: %s', self.server_capabilities.dict())
+        logger.debug(
+            'Server capabilities: %s',
+            json.dumps(self.server_capabilities, default=self._serialize_message)
+        )
 
         root_path = params.root_path
         root_uri = params.root_uri or from_fs_path(root_path)
@@ -612,9 +730,12 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
         workspace_folders = params.workspace_folders or []
         self.workspace = Workspace(root_uri, self._server.sync_kind, workspace_folders)
 
-        self.trace = Trace.Off
+        self.trace = TraceValues.Off
 
-        return InitializeResult(capabilities=self.server_capabilities)
+        return InitializeResult(
+            capabilities=self.server_capabilities,
+            server_info=self.server_info,
+        )
 
     @lsp_method(INITIALIZED)
     def lsp_initialized(self, *args) -> None:
@@ -624,10 +745,7 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
     @lsp_method(SHUTDOWN)
     def lsp_shutdown(self, *args) -> None:
         """Request from client which asks server to shutdown."""
-        for future in self._client_request_futures.values():
-            future.cancel()
-
-        for future in self._server_request_futures.values():
+        for future in self._request_futures.values():
             future.cancel()
 
         self._shutdown = True
@@ -651,7 +769,7 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
         """Puts document to the workspace."""
         self.workspace.put_document(params.text_document)
 
-    @lsp_method(SET_TRACE_NOTIFICATION)
+    @lsp_method(SET_TRACE)
     def lsp_set_trace(self, params: SetTraceParams) -> None:
         """Changes server trace value."""
         self.trace = params.value
@@ -703,14 +821,14 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
 
     def log_trace(self, message: str, verbose: Optional[str] = None) -> None:
         """Sends trace notification to the client."""
-        if self.trace == Trace.Off:
+        if self.trace == TraceValues.Off:
             return
 
         params = LogTraceParams(message=message)
-        if verbose and self.trace == Trace.Verbose:
+        if verbose and self.trace == TraceValues.Verbose:
             params.verbose = verbose
 
-        self.notify(LOG_TRACE_NOTIFICATION, params)
+        self.notify(LOG_TRACE, params)
 
     def publish_diagnostics(self, doc_uri: str, diagnostics: List[Diagnostic]) -> None:
         """Sends diagnostic notification to the client."""
