@@ -24,14 +24,17 @@ from threading import Event
 from typing import Any, Callable, List, Optional, TypeVar, Union
 
 from pygls import IS_WIN, IS_PYODIDE
+from pygls.lsp import ConfigCallbackType, ShowDocumentCallbackType
 from pygls.exceptions import PyglsError, JsonRpcException, FeatureRequestError
-from pygls.lsp.types import (ApplyWorkspaceEditResponse, ClientCapabilities, ConfigCallbackType,
-                             ConfigurationParams, Diagnostic, MessageType, RegistrationParams,
-                             ServerCapabilities, TextDocumentSyncKind, UnregistrationParams,
-                             WorkspaceEdit)
-from pygls.lsp.types.window import ShowDocumentCallbackType, ShowDocumentParams
+from lsprotocol.types import (
+    ClientCapabilities,
+    ConfigurationParams, Diagnostic, MessageType, RegistrationParams,
+    ServerCapabilities, ShowDocumentParams,
+    TextDocumentSyncKind, UnregistrationParams,
+    WorkspaceApplyEditResponse, WorkspaceEdit
+)
 from pygls.progress import Progress
-from pygls.protocol import LanguageServerProtocol, deserialize_message
+from pygls.protocol import LanguageServerProtocol, default_converter
 from pygls.workspace import Workspace
 
 if not IS_PYODIDE:
@@ -145,15 +148,17 @@ class Server:
         protocol_cls(Protocol): Protocol implementation that must be derived
                                 from `asyncio.Protocol`
 
+        converter_factory: Factory function to use when constructing a cattrs converter.
+
         loop(AbstractEventLoop): asyncio event loop
 
         max_workers(int, optional): Number of workers for `ThreadPool` and
                                     `ThreadPoolExecutor`
 
         sync_kind(TextDocumentSyncKind): Text document synchronization option
-            - NONE(0): no synchronization
-            - FULL(1): replace whole text
-            - INCREMENTAL(2): replace text within a given range
+            - None(0): no synchronization
+            - Full(1): replace whole text
+            - Incremental(2): replace text within a given range
 
     Attributes:
         _max_workers(int): Number of workers for thread pool executor
@@ -166,8 +171,8 @@ class Server:
                                                     - lazy instantiated
     """
 
-    def __init__(self, protocol_cls, loop=None, max_workers=2,
-                 sync_kind=TextDocumentSyncKind.INCREMENTAL):
+    def __init__(self, protocol_cls, converter_factory, loop=None, max_workers=2,
+                 sync_kind=TextDocumentSyncKind.Incremental):
         if not issubclass(protocol_cls, asyncio.Protocol):
             raise TypeError('Protocol class should be subclass of asyncio.Protocol')
 
@@ -191,7 +196,7 @@ class Server:
         except NotImplementedError:
             pass
 
-        self.lsp = protocol_cls(self)
+        self.lsp = protocol_cls(self, converter_factory())
 
     def shutdown(self):
         """Shutdown server."""
@@ -279,7 +284,7 @@ class Server:
             self.lsp.transport = WebSocketTransportAdapter(websocket, self.loop)
             async for message in websocket:
                 self.lsp._procedure_handler(
-                    json.loads(message, object_hook=deserialize_message)
+                    json.loads(message, object_hook=self.lsp._deserialize_message)
                 )
 
         start_server = websockets.serve(connection_made, host, port, loop=self.loop)
@@ -334,15 +339,24 @@ class LanguageServer(Server):
     exception.
     """
 
-    def __init__(self, name: str = None, version: str = None, loop=None,
-                 protocol_cls=LanguageServerProtocol, max_workers: int = 2):
+    def __init__(
+        self,
+        name: str,
+        version: str,
+        loop=None,
+        protocol_cls=LanguageServerProtocol,
+        converter_factory=default_converter,
+        max_workers: int = 2
+    ):
+
         if not issubclass(protocol_cls, LanguageServerProtocol):
             raise TypeError('Protocol class should be subclass of LanguageServerProtocol')
+
         self.name = name
         self.version = version
-        super().__init__(protocol_cls, loop, max_workers)
+        super().__init__(protocol_cls, converter_factory, loop, max_workers)
 
-    def apply_edit(self, edit: WorkspaceEdit, label: str = None) -> ApplyWorkspaceEditResponse:
+    def apply_edit(self, edit: WorkspaceEdit, label: str = None) -> WorkspaceApplyEditResponse:
         """Sends apply edit request to the client."""
         return self.lsp.apply_edit(edit, label)
 
